@@ -20,10 +20,12 @@ import glob
 import hashlib
 import importlib.util
 import itertools
+import json
 import logging
 import os
 import pkgutil
 import re
+from time import sleep
 import urllib
 from urllib import parse as urlparse
 
@@ -41,15 +43,6 @@ from cinderclient import api_versions
 from cinderclient import exceptions
 import cinderclient.extension
 
-try:
-    from eventlet import sleep
-except ImportError:
-    from time import sleep
-
-try:
-    import json
-except ImportError:
-    import simplejson as json
 
 try:
     osprofiler_web = importutils.try_import("osprofiler.web")
@@ -179,6 +172,7 @@ class SessionClient(adapter.LegacyJsonAdapter):
 
     def __init__(self, *args, **kwargs):
         apiver = kwargs.pop('api_version', None) or api_versions.APIVersion()
+        self.http_log_debug = kwargs.pop('http_log_debug', False)
         if not isinstance(apiver, api_versions.APIVersion):
             apiver = api_versions.APIVersion(str(apiver))
         if apiver.ver_minor != 0:
@@ -189,6 +183,8 @@ class SessionClient(adapter.LegacyJsonAdapter):
 
     def request(self, *args, **kwargs):
         kwargs.setdefault('authenticated', False)
+        if self.http_log_debug:
+            kwargs.setdefault('logger', self._logger)
 
         # Note(tpatil): The standard call raises errors from
         # keystoneauth, here we need to raise the cinderclient errors.
@@ -725,6 +721,7 @@ def _construct_http_client(username=None, password=None, project_id=None,
                              region_name=region_name,
                              retries=retries,
                              api_version=api_version,
+                             http_log_debug=http_log_debug,
                              **kwargs)
     else:
         # FIXME(jamielennox): username and password are now optional. Need
