@@ -22,6 +22,7 @@ import collections
 import getpass
 import logging
 import sys
+from urllib import parse as urlparse
 
 from keystoneauth1 import discover
 from keystoneauth1 import exceptions
@@ -29,11 +30,8 @@ from keystoneauth1.identity import v2 as v2_auth
 from keystoneauth1.identity import v3 as v3_auth
 from keystoneauth1 import loading
 from keystoneauth1 import session
-from oslo_utils import encodeutils
 from oslo_utils import importutils
 import requests
-import six
-import six.moves.urllib.parse as urlparse
 
 import cinderclient
 from cinderclient._i18n import _
@@ -50,7 +48,6 @@ except Exception:
 
 DEFAULT_MAJOR_OS_VOLUME_API_VERSION = "3"
 DEFAULT_CINDER_ENDPOINT_TYPE = 'publicURL'
-V2_SHELL = 'cinderclient.v2.shell'
 V3_SHELL = 'cinderclient.v3.shell'
 HINT_HELP_MSG = (" [hint: use '--os-volume-api-version' flag to show help "
                  "message for proper version]")
@@ -204,7 +201,8 @@ class OpenStackCinderShell(object):
                                               default=None),
                             help=_('Block Storage API version. '
                             'Accepts X, X.Y (where X is major and Y is minor '
-                            'part).'
+                            'part).  NOTE: this client accepts only \'3\' for '
+                            'the major version. '
                             'Default=env[OS_VOLUME_API_VERSION].'))
         parser.add_argument('--os_volume_api_version',
                             help=argparse.SUPPRESS)
@@ -225,6 +223,9 @@ class OpenStackCinderShell(object):
                             default=0,
                             help=_('Number of retries.'))
 
+        parser.set_defaults(func=self.do_help)
+        parser.set_defaults(command='')
+
         if osprofiler_profiler:
             parser.add_argument('--profile',
                                 metavar='HMAC_KEY',
@@ -330,10 +331,7 @@ class OpenStackCinderShell(object):
         parser.add_argument('--os_region_name',
                             help=argparse.SUPPRESS)
 
-        parser.add_argument(
-            '--os-token', metavar='<token>',
-            default=utils.env('OS_TOKEN'),
-            help=_('Defaults to env[OS_TOKEN].'))
+        parser.set_defaults(os_token=utils.env('OS_TOKEN'))
         parser.add_argument(
             '--os_token',
             help=argparse.SUPPRESS)
@@ -355,10 +353,7 @@ class OpenStackCinderShell(object):
         self.subcommands = {}
         subparsers = parser.add_subparsers(metavar='<subcommand>')
 
-        if version.ver_major == 3:
-            actions_module = importutils.import_module(V3_SHELL)
-        else:
-            actions_module = importutils.import_module(V2_SHELL)
+        actions_module = importutils.import_module(V3_SHELL)
 
         self._find_actions(subparsers, actions_module, version, do_help,
                            input_args)
@@ -392,7 +387,7 @@ class OpenStackCinderShell(object):
         else:
             msg = (_(" (Supported until API version %(end)s)")
                 % {"end": end_version.get_string()})
-        return six.text_type(msg)
+        return str(msg)
 
     def _find_actions(self, subparsers, actions_module, version,
                       do_help, input_args):
@@ -707,6 +702,12 @@ class OpenStackCinderShell(object):
         if not auth_session:
             auth_session = self._get_keystone_session()
 
+        # collect_timing is a keystone session option
+        if (not isinstance(auth_session, session.Session)
+                and getattr(args, 'collect_timing', False) is True):
+            raise exc.AuthorizationFailure("Provided auth plugin doesn't "
+                                           "support collect_timing option")
+
         insecure = self.options.insecure
 
         client_args = dict(
@@ -739,6 +740,10 @@ class OpenStackCinderShell(object):
         except exc.AuthorizationFailure:
             raise exc.CommandError("Unable to authorize user.")
 
+        # FIXME: this section figuring out the api version could use
+        # analysis and refactoring.  See
+        # https://review.opendev.org/c/openstack/python-cinderclient/+/766882/
+        # for some ideas.
         endpoint_api_version = None
         # Try to get the API version from the endpoint URL.  If that fails fall
         # back to trying to use what the user specified via
@@ -749,18 +754,26 @@ class OpenStackCinderShell(object):
                 self.cs.get_volume_api_version_from_endpoint()
         except exc.UnsupportedVersion:
             endpoint_api_version = options.os_volume_api_version
-            if api_version_input:
+            # FIXME: api_version_input is initialized as True at the beginning
+            # of this function and never modified
+            if api_version_input and endpoint_api_version:
                 logger.warning("Cannot determine the API version from "
                                "the endpoint URL. Falling back to the "
                                "user-specified version: %s",
                                endpoint_api_version)
-            else:
+            elif endpoint_api_version:
                 logger.warning("Cannot determine the API version from the "
                                "endpoint URL or user input. Falling back "
                                "to the default API version: %s",
                                endpoint_api_version)
+            else:
+                msg = _("Cannot determine API version.  Please specify by "
+                        "using --os-volume-api-version option.")
+                raise exc.UnsupportedVersion(msg)
 
         API_MAX_VERSION = api_versions.APIVersion(api_versions.MAX_VERSION)
+        # FIXME: the endpoint_api_version[0] can ONLY be '3' now, so the
+        # above line should probably be ripped out and this condition removed
         if endpoint_api_version[0] == '3':
             disc_client = client.Client(API_MAX_VERSION,
                                         os_username,
@@ -795,6 +808,17 @@ class OpenStackCinderShell(object):
                 print("To display trace use next command:\n"
                       "osprofiler trace show --html %s " % trace_id)
 
+            if getattr(args, 'collect_timing', False) is True:
+                self._print_timings(auth_session)
+
+    def _print_timings(self, session):
+        timings = session.get_timings()
+        utils.print_list(
+            timings,
+            fields=('method', 'url', 'seconds'),
+            sortby_index=None,
+            formatters={'seconds': lambda r: r.elapsed.total_seconds()})
+
     def _discover_client(self,
                          current_client,
                          os_api_version,
@@ -806,14 +830,9 @@ class OpenStackCinderShell(object):
                          os_auth_url,
                          client_args):
 
-        if (os_api_version.get_major_version() in
-                api_versions.DEPRECATED_VERSIONS):
-            discovered_version = api_versions.DEPRECATED_VERSION
-            os_service_type = 'volume'
-        else:
-            discovered_version = api_versions.discover_version(
-                current_client,
-                os_api_version)
+        discovered_version = api_versions.discover_version(
+            current_client,
+            os_api_version)
 
         if not os_endpoint_type:
             os_endpoint_type = DEFAULT_CINDER_ENDPOINT_TYPE
@@ -840,6 +859,11 @@ class OpenStackCinderShell(object):
             return current_client, discovered_version
 
     def _discover_service_type(self, discovered_version):
+        # FIXME: this function is either no longer needed or could use a
+        # refactoring.  The official service type is 'block-storage',
+        # which isn't even present here.  (Devstack creates 2 service
+        # types which it maps to v3: 'block-storage' and 'volumev3'.
+        # The default 'catalog_type' in tempest is 'volumev3'.)
         SERVICE_TYPES = {'1': 'volume', '2': 'volumev2', '3': 'volumev3'}
         major_version = discovered_version.get_major_version()
         service_type = SERVICE_TYPES[major_version]
@@ -1013,17 +1037,13 @@ class OpenStackHelpFormatter(argparse.HelpFormatter):
 
 def main():
     try:
-        if sys.version_info >= (3, 0):
-            OpenStackCinderShell().main(sys.argv[1:])
-        else:
-            OpenStackCinderShell().main([encodeutils.safe_decode(item)
-                                        for item in sys.argv[1:]])
+        OpenStackCinderShell().main(sys.argv[1:])
     except KeyboardInterrupt:
         print("... terminating cinder client", file=sys.stderr)
         sys.exit(130)
     except Exception as e:
         logger.debug(e, exc_info=1)
-        print("ERROR: %s" % six.text_type(e), file=sys.stderr)
+        print("ERROR: %s" % str(e), file=sys.stderr)
         sys.exit(1)
 
 
